import React from 'react';
import { SettingsContainer } from '../../components/config';
import { layerSettings, layerTypes } from './components/layers';
import { Input } from '../../..';
import { VisualizationSettingsPropTypes } from '../../common';
import { MapProps } from './mapvis';
import { nodeColorHex } from '../nodelinkvis/components/utils';

const DataLayerSettings = ({
  layer,
  settings,
  graphMetadata,
  updateSettings,
}: VisualizationSettingsPropTypes<MapProps> & { layer: keyof typeof layerTypes }) => {
  switch (layer) {
    case 'nodelink':
      return (
        <>
          {graphMetadata.nodes.labels.map((item, index) => (
            <div className="flex m-1 items-center" key={item}>
              <div className="w-3/4 mr-6">
                <EntityPill title={item} />
              </div>
              <div className="w-1/2">
                <div className={`h-5 w-5 border-2 border-sec-300`} style={{ backgroundColor: nodeColorHex(index + 1) }}></div>
              </div>
            </div>
          ))}

          <Input
            label="Enable brushing"
            type="boolean"
            value={settings.enableBrushing}
            onChange={(val) => {
              console.log('update brush', val);
              updateSettings({ enableBrushing: val as boolean });
            }}
          />
        </>
      );
    default:
      return;
  }
};

export const MapSettings = ({ settings, graphMetadata, updateSettings }: VisualizationSettingsPropTypes<MapProps>) => {
  const DataLayerSettings = settings.layer && layerSettings?.[settings.layer];

  return (
    <SettingsContainer>
      <Input
        label="Data layer"
        type="dropdown"
        inline
        value={settings.layer}
        options={Object.keys(layerTypes)}
        onChange={(val) => updateSettings({ layer: val as string })}
      />
      {DataLayerSettings && <DataLayerSettings settings={settings} graphMetadata={graphMetadata} updateSettings={updateSettings} />}
    </SettingsContainer>
  );
};
