import React from 'react';
import { CompositeLayer } from 'deck.gl';
import { LineLayer, ScatterplotLayer, TextLayer } from '@deck.gl/layers';
import { LayerProps } from '../../../mapvis.types';
import { BrushingExtension, CollisionFilterExtension } from '@deck.gl/extensions';
import { Edge, Node } from '@graphpolaris/shared/lib/data-access';

export class NodeLinkLayer extends CompositeLayer<LayerProps> {
  static type = 'NodeLink';

  updateState({ changeFlags }: { changeFlags: any }) {
    return changeFlags.propsOrDataChanged || changeFlags.somethingChanged;
  }

  renderLayers() {
    const { graph, config, getNodeLocation, selected } = this.props;

    const layers = [];

    const brushingExtension = new BrushingExtension();
    const collisionFilter = new CollisionFilterExtension();

    const brushingExtension = new BrushingExtension();
    const collisionFilter = new CollisionFilterExtension();

    graph.metaData.nodes.labels.map((label: string) => {
      layers.push(
        new ScatterplotLayer({
          id: `${label}-nodes-scatterplot`,
          visible: !config[label].hidden,
          data: graph.nodes.filter((node: Node) => node.label === label),
          pickable: true,
          filled: true,
          radiusScale: 6,
          radiusMinPixels: 7,
          radiusMaxPixels: 100,
          lineWidthMinPixels: 1,
          getPosition: (d: any) => getNodeLocation(d._id),
          getFillColor: (d: any) => config[label].color,
          getRadius: (d: any) => 5,
        }),
      );
    });

    graph.metaData.edges.labels.map((label: string) => {
      const edgeData =
        selected.length > 0 ? graph.edges.filter((edge: Edge) => selected.includes(edge.from) || selected.includes(edge.to)) : graph.edges;

      layers.push(
        new LineLayer({
          id: `${label}-edges-line`,
          data: edgeData,
          pickable: true,
          getWidth: (d: any) => config[label].width,
          getSourcePosition: (d: any) => getNodeLocation(d.from),
          getTargetPosition: (d: any) => getNodeLocation(d.to),
          getColor: (d: any) => config[d.label].color,
          radiusScale: 3000,
          brushingEnabled: config.enableBrushing,
          extensions: [brushingExtension],
        }),
      );
    });

    layers.push(
      new TextLayer({
        id: 'label-target',
        data: graph.nodes,
        getPosition: (d: any) => getNodeLocation(d._id),
        getText: (d: any) => d.id,
        getSize: 15,
        visible: true,
        getAlignmentBaseline: 'top',
        background: true,
        getPixelOffset: [10, 10],
        extensions: [collisionFilter],
        collisionEnabled: true,
        getCollisionPriority: (d: any) => d.id,
        collisionTestProps: { sizeScale: 10 },
        getRadius: 10,
        radiusUnits: 'pixels',
        collisionGroup: 'text',
      }),
    );

    return [...layers];
  }
}

NodeLinkLayer.layerName = 'NodeLink';
